
# coding: utf-8

from ...characterType import numeric, alphaNumeric
from ...row import RowElement, Row


class Trailer(Row):

    def __init__(self):
        Row.__init__(self)
        self.elements = [
            RowElement(
                index=0,
                description="Banco - Código do Banco na Compensação",
                numberOfCharacters=3,
                charactersType=numeric,
            ),
            RowElement(
                index=1,
                description="Lote - Lote de Serviço",
                numberOfCharacters=4,
                charactersType=numeric,
            ),
            RowElement(
                index=2,
                description="Registro - Tipo de Registro",
                numberOfCharacters=1,
                charactersType=numeric,
                value='5'
            ),
            RowElement(
                index=3,
                description="Complmento de Registro",
                numberOfCharacters=9,
                charactersType=alphaNumeric,
            ),
            RowElement(
                index=4,
                description="Totais - Quantidade de Registros do Lote",
                numberOfCharacters=6,
                charactersType=numeric,
            ),
            RowElement(
                index=5,
                description="Totais - Somatória dos Valores",
                numberOfCharacters=18,
                charactersType=numeric,
            ),
            RowElement(
                index=6,
                description="Complemento de Registro",
                numberOfCharacters=18,
                charactersType=numeric,
            ),
            RowElement(
                index=7,
                description="Complemento de Registro",
                numberOfCharacters=171,
                charactersType=alphaNumeric,
            ),
            RowElement(
                index=8,
                description="Ocorrências - Códigos das Ocorrências para Retorno",
                numberOfCharacters=10,
                charactersType=alphaNumeric,
            ),
        ]

    def update(self, segment):
        self.setNumberOfRegisters(num=3)
        self.setSumOfValues(sum=segment.amountInCents())

    def setNumberOfRegisters(self, num):
        self.elements[4].setValue(num)

    def setSumOfValues(self, sum):
        self.elements[5].setValue(sum)

    def setSenderBank(self, bank):
        self.elements[0].setValue(bank.bankId)

    def setPositionInLot(self, index):
        self.elements[1].setValue(index)


import re
from ...characterType import alphaNumeric, numeric


def validateInit(func):
   def wrapper(self, description, numberOfCharacters, charactersType, value=None, index=0):
       if type(numberOfCharacters) != int:
           raise ValueError("numberOfCharacters must be Integer")
       if charactersType not in [alphaNumeric, numeric]:
           raise ValueError("charactersType must be alphaNumeric or numeric")
       return func(self, description, numberOfCharacters, charactersType, value, index)
   return wrapper


def validateValue(func):
   def wrapper(self, value):
       if type(value) not in [str, int, type(None)]:
           raise ValueError("Type Value not allowed. Must be String, Integer or None. Used: %s " % str(type(value)))
       regex = {
           alphaNumeric: r'^[A-Za-z0-9\s]+$',
           numeric:      r'^[0-9]+$'
       }[self.charactersType]
       if value and not re.match(regex, str(value)):
           raise ValueError("You add a non %s value" % self.charactersType)
       return func(self, value)
   return wrapper


def validateFormatter(func):
   def wrapper(self, string, charactersType, numberOfCharacters, defaultCharacter):
       if type(numberOfCharacters) != int:
           raise ValueError("numberOfCharacters must be Integer")

       if charactersType not in [alphaNumeric, numeric]:
           raise ValueError("charactersType must be alphaNumeric or numeric")

       if not isinstance(string, str):
           raise ValueError("Type Value not allowed. Must be String. Used: %s " % str(type(string)))

       regex = {
           alphaNumeric: r'^[A-Za-z0-9\s]+$',
           numeric:      r'^[0-9]+$'
       }[charactersType]
       if string and not re.match(regex, string):
           raise ValueError("You add a non %s value" % self.charactersType)
       return func(self, string, charactersType, numberOfCharacters, defaultCharacter)
   return wrapper


from .libs.fileTools import FileTools


class File:

    def __init__(self, header, trailer):
        self.header = header
        self.lots = []
        self.trailer = trailer

    def add(self, lot):
        self.lots.append(lot)
        lot.setPositionInLot(index=len(self.lots))

    def updateTrailers(self):
        for lot in self.lots:
            lot.updateTrailer()
        self.trailer.update(lots=self.lots)

    def toString(self):
        self.header.update()
        self.updateTrailers()
        lotsToString = "\r\n".join([lot.toString() for lot in self.lots])
        centerString = "%s\r\n" % lotsToString if lotsToString else ""
        return "%s\r\n%s%s\r\n" % (self.header.toString(), centerString, self.trailer.toString())

    def setSender(self, user):
        self.header.setUser(user)
        self.header.setUserBank(user.bank)
        self.trailer.setUserBank(user.bank)

    def output(self, fileName, path="/../"):
        file = FileTools.create(name=fileName, path=path)
        file.write(self.toString())
        file.close()


from .characterType import numeric, alphaNumeric
from .libs.middlewares.row import validateInit, validateValue


class Row:

    def __init__(self):
        self.elements = []

    def toString(self):
        return "".join(map(lambda elem: elem.toString(), self.elements))


class RowElement:

    @validateInit
    def __init__(self, description, numberOfCharacters, charactersType, value, index):
        self.index = index
        self.description = description
        self.numberOfCharacters = numberOfCharacters
        self.charactersType = charactersType
        self.setValue(value)

    def value(self):
        return self.__value

    @validateValue
    def setValue(self, value):
        self.__value = value

    def toString(self):
        if self.value() is None:
            return {
                numeric:      "0" * self.numberOfCharacters,
                alphaNumeric: " " * self.numberOfCharacters
            }[self.charactersType]

        return self.__formatted(
            string=str(self.value()),
            charactersType=self.charactersType,
            numberOfCharacters=self.numberOfCharacters,
            defaultCharacter= "0" if self.charactersType == numeric else " "
        )

    def __formatted(self, string, charactersType, numberOfCharacters, defaultCharacter = " "):
        """
            This method fix the received String and a default complement according the alignment
            and cut the string if it' bigger than number of characters

            Args:
                string:             String to be completed
                charactersType:     Can be .numeric or .alphaNumeric
                numberOfCharacters: Integer that represents the max string len
                defaultCharacter:   Single string with default character to be completed if string is short
            Returns:
                String formatted
        """
        if type(string) != str:              return defaultCharacter * numberOfCharacters
        if len(string) > numberOfCharacters: return string[:numberOfCharacters]
        if charactersType == numeric:        return defaultCharacter * (numberOfCharacters - len(string)) + string
        if charactersType == alphaNumeric:   return string + defaultCharacter * (numberOfCharacters - len(string))
        return string


from ...file import File
from .header import Header
from .trailer import Trailer


class FileV83(File):

    def __init__(self):
        File.__init__(self,
            header=Header(),
            trailer=Trailer()
        )


from .occurrences import occurrences


class PaymentResponseStatus:

    success = "success"
    failed = "failed"
    scheduled = "scheduled"
    unknown = "unknown"


class PaymentResponse:

    def __init__(self, identifier, occurrences, content):
        self.identifier = identifier
        self.occurrences = occurrences
        self.content = content

    def occurrencesText(self):
        return [occurrences[occurrenceId] for occurrenceId in self.occurrences]

    def occurrencesTextAtIndex(self, index):
        occurrenceId = self.occurrences[index]
        return occurrences[occurrenceId]

    def status(self):
        if "00" in self.occurrences:
            return PaymentResponseStatus.success
        if "RJ" in self.occurrences:
            return PaymentResponseStatus.failed
        if "BD" in self.occurrences:
            return PaymentResponseStatus.scheduled
        return PaymentResponseStatus.unknown

    def contentText(self):
        return "".join(self.content)


class PaymentParser:

    @classmethod
    def parseFile(cls, file):
        lines = file.readlines()
        return cls.__parseLines(lines)

    @classmethod
    def parseText(cls, text, lineBreaker="\r\n"):
        lines = text.split(lineBreaker)[:-1]
        return cls.__parseLines(lines)

    @classmethod
    def __parseLines(cls, lines):
        result = []
        content = []
        identifier = None
        occurrences = None
        for line in lines:
            if line[7] in ["0","9"]:
                continue
            elif line[7] == "1":
                content = [line]
                identifier = None
                occurrences = None
            elif line[7] == "3":
                content.append(line)
                identifier = cls._getIdentifier(line)
                occurrences = cls._getOccurrences(line)
            elif line[7] == "5":
                content.append(line)
                result.append(PaymentResponse(
                    identifier=identifier,
                    occurrences=occurrences,
                    content=content
                ))

        return result

    @classmethod
    def _getOccurrences(cls, line):
        occurrencesString = line[230:240].replace(" ", "")
        return cls._splitString(occurrencesString)

    @classmethod
    def _splitString(cls, string):
        return [string[i:i+2] for i in range(0, len(string), 2)]

    @classmethod
    def _getIdentifier(self, line):
        return line[73:93].replace(" ", "")


from ...payment import Payment
from .header import Header
from .segmentA import SegmentA
from .trailer import Trailer


class PaymentV83(Payment):

    def __init__(self):
        Payment.__init__(self,
            header=Header(),
            segmentA=SegmentA(),
            trailer=Trailer()
        )



# coding: utf-8

from ...characterType import numeric, alphaNumeric
from ...row import RowElement, Row


class Trailer(Row):

    def __init__(self):
        Row.__init__(self)
        self.elements = [
            RowElement(
                index=0,
                description="Banco - Código do Banco na Compensação",
                numberOfCharacters=3,
                charactersType=numeric,
            ),
            RowElement(
                index=1,
                description="Lote - Lote de Serviço",
                numberOfCharacters=4,
                charactersType=numeric,
                value="9999"
            ),
            RowElement(
                index=2,
                description="Registro - Tipo de Registro",
                numberOfCharacters=1,
                charactersType=numeric,
                value="9"
            ),
            RowElement(
                index=3,
                description="CNAB - Uso Exclusivo FEBRABAN / CNAB",
                numberOfCharacters=9,
                charactersType=alphaNumeric,
            ),
            RowElement(
                index=4,
                description="Totais - Quantidade de Lotes do Arquivo",
                numberOfCharacters=6,
                charactersType=numeric,
            ),
            RowElement(
                index=5,
                description="Totais - Quantidade de Registros do Arquivo",
                numberOfCharacters=6,
                charactersType=numeric,
            ),
            RowElement(
                index=6,
                description="Complemento de Registro",
                numberOfCharacters=211,
                charactersType=alphaNumeric,
            ),
        ]

    def update(self, lots):
        self.setNumberOfLotes(num=len(lots))
        self.setNumberOfRegisters(num=2+3*len(lots))

    def setUserBank(self, bank):
        self.elements[0].setValue(bank.bankId)

    def setNumberOfLotes(self, num):
        self.elements[4].setValue(num)

    def setNumberOfRegisters(self, num):
        self.elements[5].setValue(num)



# coding: utf-8

from ...characterType import numeric, alphaNumeric
from ...row import RowElement, Row


class Header(Row):

    def __init__(self):
        Row.__init__(self)
        self.elements = [
            RowElement(
                index=0,
                description="Controle - Banco - Código do Banco na Compensação",
                numberOfCharacters=3,
                charactersType=numeric,
            ),
            RowElement(
                index=1,
                description="Controle - Lote - Lote de Serviço",
                numberOfCharacters=4,
                charactersType=numeric,
            ),
            RowElement(
                index=2,
                description="Controle - Registro - Tipo de Registro",
                numberOfCharacters=1,
                charactersType=numeric,
                value="1"
            ),
            RowElement(
                index=3,
                description="Serviço - Tipo da Operação",
                numberOfCharacters=1,
                charactersType=alphaNumeric,
                value="C" # C-Crédito
            ),
            RowElement(
                index=4,
                description="Serviço - Tipo do Pagamento",
                numberOfCharacters=2,
                charactersType=numeric
            ),
            RowElement(
                index=5,
                description="Serviço - Forma de Pagamento",
                numberOfCharacters=2,
                charactersType=numeric,
            ),
            RowElement(
                index=6,
                description="Serviço - No da Versão do Layout do Lote",
                numberOfCharacters=3,
                charactersType=numeric,
                value="040"
            ),
            RowElement(
                index=7,
                description="CNAB - Uso Exclusivo da FEBRABAN/CNAB",
                numberOfCharacters=1,
                charactersType=alphaNumeric,
            ),
            RowElement(
                index=8,
                description="Empresa - Tipo de Inscrição da Empresa",
                numberOfCharacters=1,
                charactersType=numeric,
                value='2' #1-CPF, #2-CNPJ
            ),
            RowElement(
                index=9,
                description="Empresa - Número de Inscrição da Empresa",
                numberOfCharacters=14,
                charactersType=numeric,
            ),
            RowElement(
                index=10,
                description="Empresa - Identificação do lançamento no extrato do favorecido",
                numberOfCharacters=4,
                charactersType=alphaNumeric,
            ),
            RowElement(
                index=11,
                description="Complemento de Registro",
                numberOfCharacters=16,
                charactersType=alphaNumeric,
            ),
            RowElement(
                index=12,
                description="Empresa - Agência Mantenedora da Conta",
                numberOfCharacters=5,
                charactersType=numeric,
            ),
            RowElement(
                index=13,
                description="Empresa - Dígito Verificador da Agência",
                numberOfCharacters=1,
                charactersType=alphaNumeric,
            ),
            RowElement(
                index=14,
                description="Empresa - Número da Conta Corrente",
                numberOfCharacters=12,
                charactersType=numeric,
            ),
            RowElement(
                index=15,
                description="Empresa - Dígito Verificador da Conta",
                numberOfCharacters=1,
                charactersType=alphaNumeric,
            ),
            RowElement(
                index=16,
                description="Empresa - DAC da agencia/conta debitada",
                numberOfCharacters=1,
                charactersType=numeric,
            ),
            RowElement(
                index=17,
                description="Empresa - Nome da Empresa",
                numberOfCharacters=30,
                charactersType=alphaNumeric,
            ),
            RowElement(
                index=18,
                description="Finalidade do payment do lote",
                numberOfCharacters=30,
                charactersType=alphaNumeric,
            ),
            RowElement(
                index=19,
                description="Complemento Historico C/C Debitada",
                numberOfCharacters=10,
                charactersType=alphaNumeric,
            ),
            RowElement(
                index=20,
                description="Endereço da Empresa - Logradouro",
                numberOfCharacters=30,
                charactersType=alphaNumeric,
            ),
            RowElement(
                index=21,
                description="Endereço da Empresa - Número",
                numberOfCharacters=5,
                charactersType=numeric,
            ),
            RowElement(
                index=22,
                description="Endereço da Empresa - Complemento",
                numberOfCharacters=15,
                charactersType=alphaNumeric,
            ),
            RowElement(
                index=23,
                description="Endereço da Empresa - Cidade",
                numberOfCharacters=20,
                charactersType=alphaNumeric
            ),
            RowElement(
                index=24,
                description="Endereço da Empresa - CEP",
                numberOfCharacters=8,
                charactersType=numeric,
            ),
            RowElement(
                index=25,
                description="Endereço da Empresa - Sigla do Estado",
                numberOfCharacters=2,
                charactersType=alphaNumeric,
            ),
            RowElement(
                index=26,
                description="Complemento de Registro",
                numberOfCharacters=8,
                charactersType=alphaNumeric,
            ),
            RowElement(
                index=27,
                description="Ocorrências - Códigos das Ocorrências p/ Retorno",
                numberOfCharacters=10,
                charactersType=alphaNumeric,
            )
        ]

    def setSender(self, user):
        self.elements[9].setValue(user.identifier)
        self.elements[17].setValue(user.name)

    def setSenderBank(self, bank):
        self.elements[0].setValue(bank.bankId)
        self.elements[12].setValue(bank.branchCode)
        self.elements[14].setValue(bank.accountNumber)
        self.elements[16].setValue(bank.accountVerifier)

    def setSenderAddress(self, address):
        self.elements[20].setValue(address.streetName)
        self.elements[21].setValue(address.number)
        self.elements[23].setValue(address.city)
        self.elements[24].setValue(address.zipcode)
        self.elements[25].setValue(address.state)

    def setPositionInLot(self, index):
        self.elements[1].setValue(index)

    def setInfo(self, kind, method):
        self.elements[4].setValue(kind)
        self.elements[5].setValue(method)


from .occurrences import occurrences


class SlipResponseStatus:

    registered = "registered"
    paid = "paid"
    overdue = "overdue"
    failed = "failed"
    unknown = "unknown"


class SlipResponse:

    def __init__(self, identifier, occurrences, amount):
        self.identifier = identifier
        self.occurrences = occurrences
        self.amountInCents = amount

    def occurrencesText(self):
        return [occurrences[occurrenceId] for occurrenceId in self.occurrences]

    def occurrencesTextAtIndex(self, index):
        occurrenceId = self.occurrences[index]
        return occurrences[occurrenceId]

    def status(self):
        if "02" in self.occurrences:
            return SlipResponseStatus.registered
        if "03" in self.occurrences:
            return SlipResponseStatus.failed
        if "06" in self.occurrences:
            return SlipResponseStatus.paid
        return SlipResponseStatus.unknown


class SlipParser:

    @classmethod
    def parseFile(cls, file):
        lines = file.readlines()
        return cls.__parseLines(lines)

    @classmethod
    def parseText(cls, text, lineBreaker="\r\n"):
        lines = text.split(lineBreaker)[:-1]
        return cls.__parseLines(lines)

    @classmethod
    def __parseLines(cls, lines):
        result = []
        for line in lines:
            if line[13] == "T":
                result.append(SlipResponse(
                    identifier=cls.__getIdentifier(line),
                    occurrences=cls.__getOccurrences(line),
                    amount=cls.__getAmount(line)
                ))
        return result

    @classmethod
    def __getOccurrences(cls, line):
        return [line[15:17]]

    @classmethod
    def __getIdentifier(self, line):
        return line[58:68]

    @classmethod
    def __getAmount(cls, line):
        return int(line[81:96])



# coding: utf-8

from ...characterType import numeric, alphaNumeric
from ...row import RowElement, Row


class Header(Row):

    def __init__(self):
        Row.__init__(self)
        self.elements = [
            RowElement(
                index=0,
                description="Banco - Código do Banco na Compensação",
                numberOfCharacters=3,
                charactersType=numeric,
            ),
            RowElement(
                index=1,
                description="Lote - Lote de Serviço",
                numberOfCharacters=4,
                charactersType=numeric,
                value='0000'
            ),
            RowElement(
                index=2,
                description="Registro - Tipo de Registro",
                numberOfCharacters=1,
                charactersType=numeric,
                value='0'
            ),
            RowElement(
                index=3,
                description="CNAB - Uso Exclusivo FEBRABAN / CNAB",
                numberOfCharacters=6,
                charactersType=alphaNumeric,
            ),
            RowElement(
                index=4,
                description="No DA VERSÃO DO LAYOUT DO ARQUIVO",
                numberOfCharacters=3,
                charactersType=numeric,
                value='081'
            ),
            RowElement(
                index=5,
                description="Empresa - Tipo de Inscrição da Empresa",
                numberOfCharacters=1,
                charactersType=numeric,
                value='2' # 1-CPF, 2-CNPJ
            ),
            RowElement(
                index=6,
                description="Empresa - Número de Inscrição da Empresa",
                numberOfCharacters=14,
                charactersType=numeric,
            ),
            RowElement(
                index=7,
                description="Empresa - Código do Convênio no Banco",
                numberOfCharacters=20,
                charactersType=alphaNumeric,
            ),
            RowElement(
                index=8,
                description="Empresa - Agência Mantenedora da Conta",
                numberOfCharacters=5,
                charactersType=numeric,
            ),
            RowElement(
                index=9,
                description="Empresa - Dígito Verificador da Agência",
                numberOfCharacters=1,
                charactersType=alphaNumeric,
            ),
            RowElement(
                index=10,
                description="Empresa - Número da Conta Corrente",
                numberOfCharacters=12,
                charactersType=numeric,
            ),
            RowElement(
                index=11,
                description="Empresa - Dígito Verificador da Conta",
                numberOfCharacters=1,
                charactersType=alphaNumeric,
            ),
            RowElement(
                index=12,
                description="Empresa - Dac da agencia/conta debitada",
                numberOfCharacters=1,
                charactersType=numeric,
            ),
            RowElement(
                index=13,
                description="Empresa - Nome da Empresa",
                numberOfCharacters=30,
                charactersType=alphaNumeric,
            ),
            RowElement(
                index=14,
                description="Nome do Banco",
                numberOfCharacters=30,
                charactersType=alphaNumeric,
            ),
            RowElement(
                index=15,
                description="CNAB - Uso Exclusivo FEBRABAN / CNAB",
                numberOfCharacters=10,
                charactersType=alphaNumeric,
            ),
            RowElement(
                index=16,
                description="Arquivo - Código Remessa / Retorno",
                numberOfCharacters=1,
                charactersType=numeric,
                value="1" #1- REMESSA, 2- RETORNO
            ),
            RowElement(
                index=17,
                description="Arquivo - Data de Geração do Arquivo",
                numberOfCharacters=8,
                charactersType=numeric,
            ),
            RowElement(
                index=18,
                description="Arquivo - Hora de Geração do Arquivo",
                numberOfCharacters=6,
                charactersType=numeric,
            ),
            RowElement(
                index=19,
                description="Complemento de Registro",
                numberOfCharacters=9,
                charactersType=numeric,
            ),
            RowElement(
                index=20,
                description="Arquivo - Densidade de Gravação do Arquivo",
                numberOfCharacters=5,
                charactersType=numeric,
            ),
            RowElement(
                index=21,
                description="Complemento de Registro",
                numberOfCharacters=69,
                charactersType=alphaNumeric,
            )
        ]

    def update(self):
        import datetime
        now = datetime.datetime.now()
        self.elements[17].setValue(now.strftime("%d%m%Y"))  # Dia que o arquivo foi gerado
        self.elements[18].setValue(now.strftime("%H%M%S"))  # Horario que o arquivo foi gerado

    def setUser(self, user):
        self.elements[6].setValue(user.identifier)
        self.elements[13].setValue(user.name)

    def setUserBank(self, bank):
        self.elements[0].setValue(bank.bankId)
        self.elements[8].setValue(bank.branchCode)
        self.elements[10].setValue(bank.accountNumber)
        self.elements[12].setValue(bank.accountVerifier)




# coding: utf-8

from ...characterType import numeric, alphaNumeric
from ...row import RowElement, Row


class SegmentA(Row):

    def __init__(self):
        Row.__init__(self)
        self.elements = [
            RowElement(
                index=0,
                description="Controle - Banco - Código do Banco na Compensação",
                numberOfCharacters=3,
                charactersType=numeric,
            ),
            RowElement(
                index=1,
                description="Controle - Lote - Lote de Serviço",
                numberOfCharacters=4,
                charactersType=numeric,
            ),
            RowElement(
                index=2,
                description="Controle - Registro detalhe de Lote",
                numberOfCharacters=1,
                charactersType=numeric,
                value="3"
            ),
            RowElement(
                index=3,
                description="Serviço - No Seqüencial do Registro no Lote",
                numberOfCharacters=5,
                charactersType=numeric,
                value="1"
            ),
            RowElement(
                index=4,
                description="Serviço - Código de Segmento do Reg. Detalhe",
                numberOfCharacters=1,
                charactersType=alphaNumeric,
                value="A"
            ),
            RowElement(
                index=5,
                description="Serviço - Tipo de Movimento",
                numberOfCharacters=3,
                charactersType=numeric,
            ),
            RowElement(
                index=6,
                description="Favorecido - Código da Câmara Centralizadora",
                numberOfCharacters=3,
                charactersType=numeric,
            ),
            RowElement(
                index=7,
                description="Favorecido - Código do Banco do Favorecido",
                numberOfCharacters=3,
                charactersType=numeric,
            ),
            #Note 11 - Start
            RowElement(
                index=8,
                description="Número de Agência Conta Creditada",
                numberOfCharacters=5,
                charactersType=numeric,
            ),
            RowElement(
                index=9,
                description="Complemento de Registro",
                numberOfCharacters=1,
                charactersType=alphaNumeric,
            ),
            RowElement(
                index=10,
                description="Número de Conta Corrente Credita",
                numberOfCharacters=12,
                charactersType=numeric,
            ),
            RowElement(
                index=11,
                description="Complemento de Registro",
                numberOfCharacters=1,
                charactersType=alphaNumeric,
            ),
            RowElement(
                index=12,
                description="DAC - Agencia cretitada",
                numberOfCharacters=1,
                charactersType=numeric,
            ),
            #Note 11 - End
            RowElement(
                index=13,
                description="Favorecido - Nome do Favorecido",
                numberOfCharacters=30,
                charactersType=alphaNumeric,
            ),
            RowElement(
                index=14,
                description="Crédito - No do Docum. Atribuído p/ Empresa",
                numberOfCharacters=20,
                charactersType=alphaNumeric,
            ),
            RowElement(
                index=15,
                description="Crédito - Data prevista para Pagamento",
                numberOfCharacters=8,
                charactersType=numeric,
            ),
            RowElement(
                index=16,
                description="Crédito - Tipo da Moeda",
                numberOfCharacters=3,
                charactersType=alphaNumeric,
                value="REA" #REA ou 009
            ),
            RowElement(
                index=17,
                description="Crédito - Código ISPB - Identificação da Instituição para o SPB",
                numberOfCharacters=8,
                charactersType=numeric,
            ),
            RowElement(
                index=18,
                description="Crédito - Complemento de Registro",
                numberOfCharacters=7,
                charactersType=numeric,
            ),
            RowElement(
                index=19,
                description="Crédito - Valor do Pagamento",
                numberOfCharacters=15,
                charactersType=numeric,
            ),
            RowElement(
                index=20,
                description="Crédito - No do Docum. Atribuído pelo Banco",
                numberOfCharacters=15,
                charactersType=alphaNumeric
            ),
            RowElement(
                index=21,
                description="Crédito - Complemente de Registro",
                numberOfCharacters=5,
                charactersType=alphaNumeric
            ),
            RowElement(
                index=22,
                description="Crédito - Data Real da Efetivação Pagto",
                numberOfCharacters=8,
                charactersType=numeric,
            ),
            RowElement(
                index=23,
                description="Crédito - Valor Real da Efetivação do Pagto",
                numberOfCharacters=15,
                charactersType=numeric,
            ),
            RowElement(
                index=24,
                description="Finalidade Detalhe - Informação Complementar para Hist de C/C",
                numberOfCharacters=18,
                charactersType=alphaNumeric,
            ),
            RowElement(
                index=25,
                description="Complemento de Registro",
                numberOfCharacters=2,
                charactersType=alphaNumeric,
            ),
            RowElement(
                index=26,
                description="N0 do DOC/TED/OP no Retorno",
                numberOfCharacters=6,
                charactersType=numeric,
            ),
            RowElement(
                index=27,
                description="No de Inscrição do Favorecido (CPF)/CNPJ",
                numberOfCharacters=14,
                charactersType=numeric,
            ),
            RowElement(
                index=28,
                description="Finalidade do DOC e Status do funcionario na Empresa",
                numberOfCharacters=2,
                charactersType=alphaNumeric,
            ),
            RowElement(
                index=29,
                description="Finalidade da TED",
                numberOfCharacters=5,
                charactersType=alphaNumeric,
            ),
            RowElement(
                index=30,
                description="Complemento de Registro",
                numberOfCharacters=5,
                charactersType=alphaNumeric,
            ),
            RowElement(
                index=31,
                description="Aviso ao favorecido",
                numberOfCharacters=1,
                charactersType=alphaNumeric,
            ),
            RowElement(
                index=32,
                description="Ocorrências - Códigos das Ocorrências p/ Retorno",
                numberOfCharacters=10,
                charactersType=alphaNumeric,
            )
        ]

    def amountInCents(self):
        return self.elements[19].value()

    def setSenderBank(self, bank):
        self.elements[0].setValue(bank.bankId)       # Código do banco debitado

    def setReceiver(self, user):
        self.elements[13].setValue(user.name)             # Nome FAvorecido
        self.elements[27].setValue(user.identifier)       # CPF/CNPJ Favorecido

    def setReceiverBank(self, bank):
        self.elements[7].setValue(bank.bankId)            # Código do banco do Favorecido
        self.elements[8].setValue(bank.branchCode)        # Agencia Favorecido
        self.elements[10].setValue(bank.accountNumber)    # Conta Favorecido
        self.elements[12].setValue(bank.accountVerifier)  # DAC FAvorecido

    def setAmountInCents(self, amount):
        self.elements[19].setValue(amount)                # Valor ao Favorecido

    def setPositionInLot(self, index):
        self.elements[1].setValue(index)                  # Indica index do lote

    def setScheduleDate(self, date):
        self.elements[15].setValue(date)                  # Data para transferencia

    def setInfo(self, reason):
        self.elements[28].setValue(reason)                # Motivo da transferencia

    def setIdentifier(self, identifier):
        self.elements[14].setValue(identifier)            # Identifier da empresa na transação


import os
from setuptools import setup, find_packages

with open(os.path.join(os.path.dirname(__file__), "README.md")) as readme:
    README = readme.read()

#Allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

version = "0.1.10"

setup(
    name='febraban',
    packages=find_packages(),
    include_package_data=True,
    version=version,
    description='A library to generate files that conform to the FEBRABAN formats',
    long_description=README,
    license="MIT License",
    author='Hummingbird Product Studio',
    author_email='deromir.neves@hummingbird.com.br',
    url='https://github.com/HummingbirdStudio/febraban.git',
    keywords=['febraban', 'cnab', 'transfer', 'billing', 'bank', 'cnab240', 'febraban240']
)